package de.hawhamburg.lifecycle.agent;

import com.google.gson.Gson;
import com.google.gson.JsonIOException;
import com.mongodb.MongoException;
import de.hawhamburg.lifecycle.data.Config;
import de.hawhamburg.lifecycle.data.JsonConfigOrder;
import de.hawhamburg.lifecycle.data.JsonInfoMessage;
import de.hawhamburg.livingplace.messaging.activemq.wrapper.ConnectionSettings;
import de.hawhamburg.livingplace.messaging.activemq.wrapper.LPPublisher;
import de.hawhamburg.livingplace.messaging.activemq.wrapper.LPSubscriber;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.List;
import javax.jms.JMSException;

public class LifeCycleApp implements Runnable {
    List<LifeCycleAgent> agentList;
    private boolean running;
    private boolean connected;


    /**
     * sending an info message and die after that .
     * @author Jens Ellenberg, Jan Menzel
     *
     */
    private class SendInfo implements Runnable {

        private JsonInfoMessage infoMessage;

        public SendInfo(final JsonInfoMessage sendInfoMessage) {
            this.infoMessage = sendInfoMessage;
        }

        public void run() {
            ConnectionSettings settings = new ConnectionSettings();
            settings.amq_ip = Config.ACTIVEMQIP;
            settings.mongo_ip = Config.MONGODBIP;
            connected = false;
            LPPublisher sender = null;
            do {
                try {
                    sender = new LPPublisher("GlobalInfoTopic", settings);
                    connected = true;
                } catch (UnknownHostException e) {
                    System.err.println("host fail, waiting "
                                       + Config.WAIT_BEFORE_RECONNECT + "ms");
                    try {
                        Thread.sleep(Config.WAIT_BEFORE_RECONNECT);
                    } catch (InterruptedException e1) {
                        e1.printStackTrace();
                    }
                } catch (MongoException e) {
                    System.err.println("mongoDb fail, waiting "
                                       + Config.WAIT_BEFORE_RECONNECT + "ms");
                    try {
                        Thread.sleep(Config.WAIT_BEFORE_RECONNECT);
                    } catch (InterruptedException e1) {
                        e1.printStackTrace();
                    }
                } catch (JMSException e) {
                    System.err.println("connection fail, waiting "
                                       + Config.WAIT_BEFORE_RECONNECT + "ms");
                    try {
                        Thread.sleep(Config.WAIT_BEFORE_RECONNECT);
                    } catch (InterruptedException e1) {
                        e1.printStackTrace();
                    }
                }
            } while (!connected && running);

            Gson g = new Gson();
            sender.publish(g.toJson(infoMessage));
            sender.disconnect();
        }
    }

    public LifeCycleApp() {
        running = true;
        agentList = new ArrayList<LifeCycleAgent>();
    }

    /**
     * add a agent to the listener.
     * @param aAgent an life cycle agent
     */
    public final void setLifeCycleAgent(final LifeCycleAgent aAgent) {
        agentList.add(aAgent);
    }

    public final void run() {
        ConnectionSettings settings = new ConnectionSettings();
        settings.amq_ip = Config.ACTIVEMQIP;
        settings.mongo_ip = Config.MONGODBIP;
        LPSubscriber client = null;
        do {
            try {
                connected = false;
                client = new LPSubscriber("GlobalControlTopic", settings);
                connected = true;
            } catch (JMSException e) {
                System.err.println("connection fail, waiting "
                                   + Config.WAIT_BEFORE_RECONNECT + "ms");
                try {
                    Thread.sleep(Config.WAIT_BEFORE_RECONNECT);
                } catch (InterruptedException e1) {
                    e1.printStackTrace();
                }
            }
        } while (!connected);

        Gson g = new Gson();
        String message = "";
        while (running) {
            try {
                message = client.subscribeBlockingWithTimeout(Config.AMQ_READ_TIMEOUT);
                // if there is a message
                if (message != null) {
                    JsonConfigOrder order = g.fromJson(message, JsonConfigOrder.class);

                    switch (order.order) {
                        case start:
                            for (LifeCycleAgent agent : agentList) {
                                if (order.Id.equals(agent.getID())) {
                                    agent.start();
                                    new Thread(new SendInfo(agent.getInfoMessage())).start();
                                }
                            }
                            break;
                        case info:
                            for (LifeCycleAgent agent : agentList) {
                                new Thread(new SendInfo(agent.getInfoMessage())).start();
                            }
                            break;
                        case sleep:
                            for (LifeCycleAgent agent : agentList) {
                                if (order.Id.equals(agent.getID())) {
                                    agent.sleep();
                                    new Thread(new SendInfo(agent.getInfoMessage())).start();
                                }
                            }
                            break;
                        case stop:
                            for (LifeCycleAgent agent : agentList) {
                                if (order.Id.equals(agent.getID())) {
                                    agent.stop();
                                    new Thread(new SendInfo(agent.getInfoMessage())).start();
                                    running = false;
                                }
                            }
                            break;
                        default:
                            System.err.println("unknown Message: " + order);
                            break;
                    }
                }

            } catch (InterruptedException e) {
                e.printStackTrace();
            } catch (JMSException e) {
                e.printStackTrace();
            } catch (JsonIOException e) {
                System.err.println("Gson failed to read: " + message);
            }
        }
        client.disconnect();
    }

    /**
     * stop the running App.
     */
    public final void stop() {
        running = false;
    }

}
