package de.hawhamburg.reachability.sensor;

/**
 * Common default settings for sensors related to the reachability project.<br><br>
 * 
 * This configuration class contains common default settings for different sensors in common with
 * the reachability project. Generally the parameters are related to standard configuration file access.
 * 
 * @author  {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_AUTHOR} 
 * @version {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_VERSION}
 */
public final class Settings {

    /**
     * The path prefix to the XML file where the configuration for the sensors is defined.  
     */
    public static final String SENSOR_CONFIGURATION_PATH    = "resource/sensor/";
    
    /**
     * The xPath within the sensor configuration file which points to the request topic.  
     */
    public static final String SENSOR_REQUEST_TOPIC_XPATH   = "/configuration/topics/topic[@type='request']";
    
    /**
     * The xPath within the sensor configuration file which points to the result topic.  
     */
    public static final String SENSOR_RESULT_TOPIC_XPATH    = "/configuration/topics/topic[@type='result']";
    
    /**
     * The xPath within the sensor configuration file which points to the delay time.  
     */
    public static final String SENSOR_DELAY_TIME_XPATH      = "/configuration/checkDelayTime";
    
    /**
     * The xPath within the sensor configuration file which points to a certain local configuration parameter.  
     */
    public static final String SENSOR_LOCAL_CONF_XPATH      = "/configuration/localConf/set";
    
    /**
     * Private default constructor to prevent that instances of this class will be created.
     */
    private Settings() { }
}
